@@ -28,6 +28,7 @@ from __future__ import print_function
 
 import argparse
 import getpass
+import inspect
 import logging
 import ovirtsdk4 as sdk
 import ovirtsdk4.types as types
@@ -35,7 +36,7 @@ import time
 
 from ovirt_imageio import client
 
-logging.basicConfig(level=logging.DEBUG, filename='example.log')
+from helpers import imagetransfer
 
 # This example will connect to the server and download the data
 # of the disk to a local file.
@@ -93,6 +94,20 @@ def parse_args():
         action="store_true",
         help="download via proxy on the engine host (less efficient)")
 
+    parser.add_argument(
+        "--max-workers",
+        type=int,
+        default=4,
+        help="maximum number of workers to use for download. The default "
+             "(4) improves performance when downloading a single disk. "
+             "You may want to use lower number if you download many disks "
+             "in the same time.")
+
+    parser.add_argument(
+        "--debug",
+        action="store_true",
+        help="log debug level messages to example.log")
+
     return parser.parse_args()
 
 
@@ -106,6 +121,12 @@ def read_password(args):
 
 args = parse_args()
 
+logging.basicConfig(
+    level=logging.DEBUG if args.debug else logging.INFO,
+    filename="example.log",
+    format="%(asctime)s %(levelname)-7s (%(threadName)s) [%(name)s] %(message)s"
+)
+
 # Create the connection to the server:
 print("Connecting...")
 
@@ -114,7 +135,7 @@ connection = sdk.Connection(
     username=args.username,
     password=read_password(args),
     ca_file=args.cafile,
-    debug=True,
+    debug=args.debug,
     log=logging.getLogger(),
 )
 
@@ -128,56 +149,43 @@ disks_service = connection.system_service().disks_service()
 disk_service = disks_service.disk_service(args.disk_uuid)
 disk = disk_service.get()
 
+# Find a host for this transfer. This is an optional step allowing optimizing
+# the transfer using unix socket when running this code on a oVirt hypervisor
+# in the same data center.
+sd_id = disk.storage_domains[0].id
+sds_service = system_service.storage_domains_service()
+storage_domain = sds_service.storage_domain_service(sd_id).get()
+host = imagetransfer.find_host(connection, storage_domain.name)
+
 print("Creating image transfer...")
 
-# Get a reference to the service that manages the image
-# transfer that was added in the previous step:
-transfers_service = system_service.image_transfers_service()
+transfer = imagetransfer.create_transfer(
+    connection, disk, types.ImageTransferDirection.DOWNLOAD, host=host)
 
-# Add a new image transfer:
-transfer = transfers_service.add(
-    types.ImageTransfer(
-        image=types.Image(
-            id=disk.id
-        ),
-        direction=types.ImageTransferDirection.DOWNLOAD,
-
-        # format=raw enables the NBD backend, enbaling:
-        # - Download raw guest data, regardless of the disk format.
-        # - Collapsed qcow2 chains to single raw file.
-        # - Extents reporting for qcow2 images and raw images on file storage,
-        #   speeding up the transfer.
-        format=types.DiskFormat.RAW,
-    )
-)
-
-# Get reference to the created transfer service:
-transfer_service = transfers_service.image_transfer_service(transfer.id)
-
-# After adding a new transfer for the disk, the transfer's status will be INITIALIZING.
-# Wait until the init phase is over. The actual transfer can start when its status is "Transferring".
-while transfer.phase == types.ImageTransferPhase.INITIALIZING:
-    time.sleep(1)
-    transfer = transfer_service.get()
-
-# You can use the transfer to locate logs for this transfer.
 print("Transfer ID: %s" % transfer.id)
+print("Transfer host name: %s" % transfer.host.name)
 
-# Fetch the transfer host name. This is very useful for troubleshooting.
-hosts_service = connection.system_service().hosts_service()
-host_service = hosts_service.host_service(transfer.host.id)
-transfer_host = host_service.get()
+# At this stage, the SDK granted the permission to start transferring the disk, and the
+# user should choose its preferred tool for doing it. We use the recommended
+# way, ovirt-imageio client library.
 
-print("Transfer host: %s" % transfer_host.name)
+extra_args = {}
+
+parameters = inspect.signature(client.download).parameters
+
+# Use multiple workers to speed up the download.
+if "max_workers" in parameters:
+        extra_args["max_workers"] = args.max_workers
 
 if args.use_proxy:
     download_url = transfer.proxy_url
 else:
     download_url = transfer.transfer_url
 
-# At this stage, the SDK granted the permission to start transferring the disk, and the
-# user should choose its preferred tool for doing it. We use the recommended
-# way, ovirt-imageio client library.
+    # Use fallback to proxy_url if feature is available. Download will use the
+    # proxy_url if transfer_url is not accessible.
+    if "proxy_url" in parameters:
+        extra_args["proxy_url"] = transfer.proxy_url
 
 print("Downloading image...")
 
@@ -189,11 +197,11 @@ try:
             args.cafile,
             fmt=args.format,
             secure=args.secure,
-            progress=pb)
+            progress=pb,
+            **extra_args)
 finally:
-    # Finalize the session.
     print("Finalizing image transfer...")
-    transfer_service.finalize()
+    imagetransfer.finalize_transfer(connection, transfer, disk)
 
 # Close the connection to the server:
 connection.close()
