@@ -31,6 +31,7 @@ from __future__ import print_function
 
 import argparse
 import getpass
+import inspect
 import json
 import logging
 import os
@@ -42,7 +43,8 @@ import time
 
 from ovirt_imageio import client
 
-logging.basicConfig(level=logging.DEBUG, filename='example.log')
+from helpers import imagetransfer
+
 
 def parse_args():
     parser = argparse.ArgumentParser(description="Upload images")
@@ -77,13 +79,17 @@ def parse_args():
         action="store_true",
         help="create sparse disk. Cannot be used with raw format on block storage.")
 
+    parser.add_argument(
+        "--enable-backup",
+        action="store_true",
+        help="creates a disk that can be used for incremental backup. "
+             "Allowed for disk with qcow2 format only")
+
     parser.add_argument(
         "--sd-name",
         required=True,
         help="name of the storage domain.")
 
-    # Note: unix socket works only when running this tool on the same host serving
-    # the image.
     parser.add_argument(
         "-c", "--cafile",
         help="path to oVirt engine certificate for verifying server.")
@@ -103,8 +109,23 @@ def parse_args():
         action="store_true",
         help="upload via proxy on the engine host (less efficient)")
 
+    parser.add_argument(
+        "--max-workers",
+        type=int,
+        default=4,
+        help="Maximum number of workers to use for upload. The default "
+             "(4) improves performance when uploading a single disk. "
+             "You may want to use lower number if you upload many disks "
+             "in the same time.")
+
+    parser.add_argument(
+        "--debug",
+        action="store_true",
+        help="log debug level messages to example.log")
+
     return parser.parse_args()
 
+
 def get_image_info(filename):
     print("Checking image...")
 
@@ -179,6 +200,12 @@ def get_disk_info(args, image_info):
 
 args = parse_args()
 
+logging.basicConfig(
+    level=logging.DEBUG if args.debug else logging.INFO,
+    filename="example.log",
+    format="%(asctime)s %(levelname)-7s (%(threadName)s) [%(name)s] %(message)s"
+)
+
 # Get image and disk info using qemu-img
 image_info = get_image_info(args.filename)
 disk_info = get_disk_info(args, image_info)
@@ -189,6 +216,7 @@ print("Disk content type: %s" % disk_info["content_type"])
 print("Disk provisioned size: %s" % disk_info["provisioned_size"])
 print("Disk initial size: %s" % disk_info["initial_size"])
 print("Disk name: %s" % disk_info["name"])
+print("Disk backup: %s" % args.enable_backup)
 
 # This example will connect to the server and create a new `floating`
 # disk, one that isn't attached to any virtual machine.
@@ -209,7 +237,7 @@ connection = sdk.Connection(
     username=args.username,
     password=password,
     ca_file=args.cafile,
-    debug=True,
+    debug=args.debug,
     log=logging.getLogger(),
 )
 
@@ -228,6 +256,7 @@ disk = disks_service.add(
         initial_size=disk_info["initial_size"],
         provisioned_size=disk_info["provisioned_size"],
         sparse=args.disk_sparse,
+        backup=types.DiskBackup.INCREMENTAL if args.enable_backup else None,
         storage_domains=[
             types.StorageDomain(
                 name=args.sd_name
@@ -245,67 +274,56 @@ while True:
     if disk.status == types.DiskStatus.OK:
         break
 
-print("Disk id: %s" % disk.id)
+print("Disk ID: %s" % disk.id)
 
 print("Creating image transfer...")
 
-# Get a reference to the service that manages the image
-# transfer that was added in the previous step:
-transfers_service = system_service.image_transfers_service()
+# Find a host for this transfer. This is an optional step allowing optimizing
+# the transfer using unix socket when running this code on a oVirt hypervisor
+# in the same data center.
+host = imagetransfer.find_host(connection, args.sd_name)
 
-# Add a new image transfer:
-transfer = transfers_service.add(
-    types.ImageTransfer(
-        image=types.Image(
-            id=disk.id
-        ),
-        # Use raw format to enable NBD backend, supporting on-the-fly image
-        # format conversion.
-        format=types.DiskFormat.RAW,
-     )
-)
+transfer = imagetransfer.create_transfer(connection, disk,
+    types.ImageTransferDirection.UPLOAD, host=host)
 
-# Get reference to the created transfer service:
-transfer_service = transfers_service.image_transfer_service(transfer.id)
-
-# After adding a new transfer for the disk, the transfer's status will be INITIALIZING.
-# Wait until the init phase is over. The actual transfer can start when its status is "Transferring".
-while transfer.phase == types.ImageTransferPhase.INITIALIZING:
-    time.sleep(1)
-    transfer = transfer_service.get()
-
-# You can use the transfer id to locate logs for this transfer.
 print("Transfer ID: %s" % transfer.id)
-
-# Fetch the transfer host name. This is very useful for troubleshooting.
-hosts_service = connection.system_service().hosts_service()
-host_service = hosts_service.host_service(transfer.host.id)
-transfer_host = host_service.get()
-
-print("Transfer host: %s" % transfer_host.name)
+print("Transfer host name: %s" % transfer.host.name)
 
 # At this stage, the SDK granted the permission to start transferring the disk, and the
 # user should choose its preferred tool for doing it. We use the recommended
 # way, ovirt-imageio client library.
 
+extra_args = {}
+
+parameters = inspect.signature(client.download).parameters
+
+# Use multiple workers to speed up the upload.
+if "max_workers" in parameters:
+        extra_args["max_workers"] = args.max_workers
+
 if args.use_proxy:
-    destination_url = transfer.proxy_url
+    upload_url = transfer.proxy_url
 else:
-    destination_url = transfer.transfer_url
+    upload_url = transfer.transfer_url
+
+    # Use fallback to proxy_url if feature is available. Upload will use the
+    # proxy_url if transfer_url is not accessible.
+    if "proxy_url" in parameters:
+        extra_args["proxy_url"] = transfer.proxy_url
 
 print("Uploading image...")
 
 with client.ProgressBar() as pb:
     client.upload(
         args.filename,
-        destination_url,
+        upload_url,
         args.cafile,
         secure=args.secure,
-        progress=pb)
+        progress=pb,
+        **extra_args)
 
 print("Finalizing image transfer...")
-# Successful cleanup
-transfer_service.finalize()
+imagetransfer.finalize_transfer(connection, transfer, disk)
 connection.close()
 
 print("Upload completed successfully")
